import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import lognorm
from tc_python import *

"""
This example simulates the kinetics of precipitation of CEMENTITE from a BCC_A2 solution phase and shows the grain growth results
for two cases: with and without Zener pining. A log normal distribution is applied for the matrix grain size.
"""

cut_off = 1e-5
mu_log_norm = 2.0E-6
sigma_log_norm = 0.5
x_log_norm = np.linspace(lognorm.ppf(cut_off, s=sigma_log_norm, scale=mu_log_norm),
                         lognorm.ppf(1 - cut_off, s=sigma_log_norm, scale=mu_log_norm), 10000)
pdf_lognorm = lognorm.pdf(x_log_norm, s=sigma_log_norm, scale=mu_log_norm)

exp_one_x = [0.167, 0.583, 2.5, 4.5, 7.0, 11.0, 21.0, 35.0]
exp_one_y = [3.3, 3.6, 4.4, 4.8, 5.7, 5.4, 6.7, 7.1]

sim_time = 126000


def simulate(system_input, has_pinner):
    grain_size_distribution = GrainSizeDistribution()
    for idx, val in enumerate(x_log_norm):
        grain_size_distribution.add_radius_and_number_density(val, pdf_lognorm[idx])
    grain_growth = GrainGrowth(grain_size_distribution)
    if has_pinner:
        grain_growth.enable_zener_pinning()
    else:
        grain_growth.disable_zener_pinning()
    grain_growth.set_grain_boundary_mobility_activation_energy(242000.0)
    grain_growth.set_grain_boundary_energy(0.5)
    grain_growth.set_grain_boundary_mobility_pre_factor(0.004)

    matrix_phase = (MatrixPhase("BCC_A2")
                    .with_grain_growth_model(grain_growth)
                    .add_precipitate_phase(PrecipitatePhase("CEMENTITE")
                                           .set_nucleation_in_bulk()
                                           .set_interfacial_energy_estimation_prefactor(0.15))
                    )
    calculation = (system_input.with_matrix_phase(matrix_phase))

    sim_results = calculation.calculate()
    return sim_results


def plot_results(no_pinning_res, with_pinning_res):
    fig, (ax1) = plt.subplots(1, 1)
    time_without_pining, mean_grain_size_without_pining = no_pinning_res.get_grain_mean_radius()
    time_with_pining, mean_grain_size_with_pining = with_pinning_res.get_grain_mean_radius()
    ax1.plot(np.array(time_without_pining) / 3600, 1e6 * np.array(mean_grain_size_without_pining),
             label='Mean radius of BCC_A2 (No pinning)')
    ax1.plot(np.array(time_with_pining) / 3600, 1e6 * np.array(mean_grain_size_with_pining),
             label='Mean radius of BCC_A2 (With pinning)')
    exp_one_x_simulated = []
    exp_one_y_simulated = []
    for idx, val in enumerate(exp_one_x):
        if val <= sim_time / 3600:
            exp_one_x_simulated.append(exp_one_x[idx])
            exp_one_y_simulated.append(exp_one_y[idx])
    if len(exp_one_x_simulated) > 0:
        ax1.plot(exp_one_x_simulated, exp_one_y_simulated, "^", label="Experiment")
    fig.suptitle('With/without Zener pinning', fontsize=16)
    ax1.set_xlabel('Time [h]')
    ax1.set_ylabel('Length [μm]')
    ax1.legend(loc='best')

    plt.show()


with TCPython() as start:
    start.set_cache_folder(os.path.basename(__file__) + "_cache")
    elements = ["Fe", "C"]
    system = (start.select_thermodynamic_and_kinetic_databases_with_elements("FEDEMO", "MFEDEMO", elements).get_system()
              .with_isothermal_precipitation_calculation()
              .set_composition_unit(CompositionUnit.MASS_PERCENT)
              .set_composition("C", 0.2)
              .set_temperature(995.15)
              .set_simulation_time(sim_time))

    no_pinning_results = simulate(system, False)
    with_pinning_results = simulate(system, True)

    plot_results(no_pinning_results, with_pinning_results)
